const tkz = @import("tokenize.zig");
const hlp = @import("helpers.zig");
const std = @import("std");

pub const TokenClass = enum {
    symbol,
    string,
    comment,
    docstring,
    number,
    nil,
    t,
    list_start,
    list_lazy_start,
    list_end,
    operator,
};

pub const ClassifiedToken = struct {
    tok: tkz.Token,
    cls: TokenClass,
};

const operators = std.ComptimeStringMap(void, .{
    .{"->"},
    .{"~>"},
    .{"/="},
    .{"*="},
    .{"-="},
    .{"+="},
    .{"||"},
    .{"&&"},
    .{"="},
    .{"+"},
    .{"-"},
    .{"*"},
    .{"/"},
    .{"~"},
    .{"%"},
    .{"@keys"},
    .{"@rest"},
});

const numbers = std.ComptimeStringMap(void, .{
    .{"0"},
    .{"1"},
    .{"2"},
    .{"3"},
    .{"4"},
    .{"5"},
    .{"6"},
    .{"7"},
    .{"8"},
    .{"9"},
});

fn classify(tok: tkz.Token) ClassifiedToken {
    return ClassifiedToken{
        .tok = tok,
        .cls = if (operators.has(tok.value))
            TokenClass.operator
        else if (std.mem.eql(u8, "'(", tok.value))
            TokenClass.list_lazy_start
        else if (std.mem.eql(u8, "(", tok.value))
            TokenClass.list_start
        else if (std.mem.eql(u8, ")", tok.value))
            TokenClass.list_end
        else if (std.mem.eql(u8, "\"", tok.value[0..1]))
            TokenClass.string
        else if (std.mem.eql(u8, "nil", tok.value))
            TokenClass.nil
        else if (std.mem.eql(u8, "t", tok.value))
            TokenClass.t
        else if (numbers.has(tok.value[0..1]))
            TokenClass.number
        else if (std.mem.eql(u8, ";", tok.value[0..1]))
            if (tok.value.len >= 3 and std.mem.eql(u8, ";;;", tok.value[0..3])) TokenClass.docstring else TokenClass.comment
        else
            TokenClass.symbol,
    };
}

pub fn classifyTokens(toks: []const tkz.Token, allocator: std.mem.Allocator) !std.ArrayList(ClassifiedToken) {
    var ctoks = std.ArrayList(ClassifiedToken).init(allocator);

    for (toks) |tok| {
        try ctoks.append(classify(tok));
    }

    return ctoks;
}
