const std = @import("std");
const lsp_types = @import("lsfw/src/types.zig");
const lsp = @import("lsfw/src/lsp.zig");
const lsp_doc = @import("lsfw/src/document.zig");
const lsp_log = @import("lsfw/src/logger.zig");
const tkz = @import("tokenize.zig");
const cls = @import("classifier.zig");

const State = struct { symbols: std.ArrayList(cls.ClassifiedToken) };
const Lsp = lsp.Lsp(State);
const Scope = enum { hi };

fn handleHover(allocator: std.mem.Allocator, ctx: *Lsp.Context, pos: lsp_types.Position) ?[]const u8 {
    if (null == ctx.state) {
        lsp_log.notify(.info, "could not find token under cursor (at {})", .{pos});
        return null;
    } else if (0 == ctx.state.?.symbols.items.len) {
        handleDocOpen(allocator, ctx);
    }
    lsp_log.notify(.err, "{}", .{ctx.state.?.symbols});
    // for (ctx.state.?.symbols.items) |tok| {
    //     if (tok.tok.line == pos.line and tok.tok.char <= pos.character and (tok.tok.char + tok.tok.value.len) >= pos.character) {
    //         lsp_log.notify(.info, "{}", .{tok});
    //         break;
    //     }
    // }

    return null;
}

fn handleCompletion(allocator: std.mem.Allocator, context: *Lsp.Context, position: lsp_types.Position) ?lsp_types.CompletionList {
    _ = context;
    _ = position;
    var completions = std.ArrayList(lsp_types.CompletionItem).init(allocator);
    if (std.mem.Allocator.Error.OutOfMemory == completions.append(.{
        .label = "(procedure)",
        .insertText = "(procedure ${1:func_name}($2)\n\n)",
        .insertTextFormat = .Snippet,
        .kind = .Function,
    })) {
        return null;
    }

    return .{ .items = completions.items };
}

fn handleDocOpen(allocator: std.mem.Allocator, context: *Lsp.Context) void {
    lsp_log.notify(.err, "opened doc {s}", .{context.document.uri});
    const content = context.document.text;
    const toks = tkz.tokenizeContent(content, allocator) catch unreachable;
    // const toks = std.ArrayList(tkz.Token).init(allocator);
    lsp_log.notify(.err, "toks {}", .{toks});
    // defer toks.deinit();
    const ctoks = cls.classifyTokens(toks.items, allocator) catch unreachable;
    lsp_log.notify(.err, "ctoks {}", .{ctoks});
    // defer ctoks.deinit();
    // const ast = try stx.generateSyntaxTree(ctoks);

    lsp_log.notify(.info, "opened {s}, found {d} tokens", .{ context.document.uri, ctoks.items.len });
    if (context.state != null) {
        context.state.?.symbols.deinit();
    }
    context.state = .{
        .symbols = std.ArrayList(cls.ClassifiedToken).init(allocator),
    };
}
fn handleDocChanged(allocator: std.mem.Allocator, context: *Lsp.Context, _: []lsp_types.ChangeEvent) void {
    handleDocOpen(allocator, context);
}

fn handleDocClose(_: std.mem.Allocator, _: *Lsp.Context) void {}

pub fn start() !u8 {
    const descr = lsp_types.ServerData{
        .serverInfo = .{
            .name = "skill lsp",
            .version = "0.1.0",
        },
    };

    var gpa = std.heap.GeneralPurposeAllocator(.{}){};
    var server = Lsp.init(gpa.allocator(), descr);

    server.registerHoverCallback(handleHover);
    server.registerCompletionCallback(handleCompletion);
    server.registerDocOpenCallback(handleDocOpen);
    server.registerDocChangeCallback(handleDocChanged);
    server.registerDocCloseCallback(handleDocClose);
    return server.start();
}
