const std = @import("std");
const cls = @import("classifier.zig");

pub const SyntaxNode = struct {
    ctok: cls.ClassifiedToken,
    nodes: ?std.ArrayList(SyntaxNode),
};

pub fn generateSyntaxTree(ctoks: std.ArrayList(cls.ClassifiedToken)) !std.ArrayList(SyntaxNode) {
    var nodes = std.ArrayList(SyntaxNode).init(std.heap.page_allocator);
    var actives = std.ArrayList(SyntaxNode).init(std.heap.page_allocator);

    for (ctoks.items) |ctok| {
        switch (ctok.cls) {
            cls.TokenClass.comment, cls.TokenClass.docstring => {
                try nodes.append(.{
                    .ctok = ctok,
                    .nodes = null,
                });
            },
            cls.TokenClass.list_start, cls.TokenClass.list_lazy_start => {
                try actives.append(.{
                    .ctok = ctok,
                    .nodes = std.ArrayList(SyntaxNode).init(std.heap.page_allocator),
                });
            },
            cls.TokenClass.list_end => {
                if (actives.items.len > 0) {
                    try nodes.append(actives.pop());
                } else {
                    std.debug.print("{}\n", .{actives});
                }
            },
            else => {
                const active_top = actives.popOrNull();
                if (active_top != null) {
                    var active = active_top.?;
                    var actives_nodes: std.ArrayList(SyntaxNode) = undefined;
                    if (active.nodes != null) {
                        actives_nodes = active.nodes.?;
                    } else {
                        active.nodes = std.ArrayList(SyntaxNode).init(std.heap.page_allocator);
                        actives_nodes = active.nodes.?;
                    }
                    try actives_nodes.append(.{
                        .ctok = ctok,
                        .nodes = null,
                    });
                } else {}
            },
        }
    }

    return nodes;
}
