const std = @import("std");
const lsp = @import("lsfw/src/lsp.zig");

pub const Token = struct {
    /// 0-based index of token start in whole file
    start: usize,
    /// 1-based line numbert token starts at
    line: usize,
    /// 1-based char numbert token starts at in line
    char: usize,

    value: []const u8,
};

const TokenizationError = error{InvalidKeyword};

pub fn tokenizeContent(content: []u8, allocator: std.mem.Allocator) !std.ArrayList(Token) {
    var toks = std.ArrayList(Token).init(allocator);
    var lines = std.ArrayList(usize).init(allocator);
    defer lines.deinit();

    var index: usize = 0;
    while (index < content.len) {
        var l: usize = 1;
        const char = content[index];
        _ = switch (char) {
            '\n' => {
                try lines.append(index);
                index += l;
                continue;
            },
            ';' => {
                while (switch (content[index + l]) {
                    '\n' => false,
                    else => true,
                }) : (l += 1) {}
            },
            '"' => {
                while (switch (content[index + l]) {
                    '"' => (content[index + l - 1] == '\\'),
                    else => true,
                }) : (l += 1) {}
                l += 1;
            },
            'a'...'z', 'A'...'Z', '_' => {
                while (switch (content[index + l]) {
                    'a'...'z', 'A'...'Z', '0'...'9', '_' => true,
                    else => false,
                }) : (l += 1) {}
            },
            '0'...'9' => {
                while (switch (content[index + l]) {
                    '0'...'9', '.', 'e' => true,
                    else => false,
                }) : (l += 1) {}
            },
            '+', '-', '~', '*', '/', '%', '<', '>', '=', '?', '|', '&', '(', ')', '\'' => {
                for ([_]*const [2]u8{ "->", "~>", "||", "&&", "/=", "*=", "+=", "-=", "'(" }) |op| {
                    if (std.mem.eql(u8, op, content[index .. index + 2])) {
                        l = 2;
                        break;
                    }
                }
            },
            '@' => {
                while (switch (content[index + l]) {
                    'a'...'z', 'A'...'Z', '_', '0'...'9' => true,
                    else => false,
                }) : (l += 1) {}

                if (std.mem.eql(u8, "@keys", content[index .. index + l])) {} else if (std.mem.eql(u8, "@rest", content[index .. index + l])) {} else {
                    std.debug.print("line={d}, char={d}\n", .{
                        .line = lines.items.len,
                        .char = switch (lines.items.len) {
                            0 => index,
                            else => index - lines.items[lines.items.len - 1],
                        },
                    });
                }
            },
            else => {
                index += l;
                continue;
            },
        };
        try toks.append(.{
            .start = index,
            .value = try allocator.dupe(u8, content[index .. index + l]),
            .line = lines.items.len,
            .char = switch (lines.items.len) {
                0 => index,
                else => index - lines.items[lines.items.len - 1],
            },
        });
        index += l;
    }
    lsp.logger.notify(.err, "done with initial tokenization, generated {d} tokens", .{toks.items.len});
    return toks;
}
